# Built in Libraries
import sys
import logging
import pdb
import os
from os.path import dirname, abspath, join
from importlib import import_module
import logging

# 3rd Party Libraries
import pickle
import pandas as pd

# Alteryx Libraries
import AlteryxPythonSDK as Sdk

# Custom libraries
from snakeplane.plugin_factory import PluginFactory

# Initialization of the plug in factory, used for making the AyxPlugin class
factory = PluginFactory("AzureML_Score")


@factory.initialize_plugin
def init(workflow_config, user_data, logger):
    # Grab UI Data
    user_data.model_name = workflow_config.get("name")
    user_data.subscription_id = workflow_config.get("subscription_id")
    user_data.resource_group = workflow_config.get("resource_group")
    user_data.workspace_name = workflow_config.get("workspace_name")

    initSuccess = True

    # Check for required Values
    if user_data.model_name is None:
        logger.display_error_msg("Model requires a name")
        initSuccess = False

    if user_data.subscription_id is None:
        logger.display_error_msg("Subscription ID is Required")
        initSuccess = False

    if user_data.resource_group is None:
        logger.display_error_msg("Resource Group is Required")
        initSuccess = False

    if user_data.workspace_name is None:
        logger.display_error_msg("Workspace Name is Required")
        initSuccess = False

    return initSuccess


@factory.process_data(mode="batch", input_type="dataframe")
def process_data(input_mgr, output_mgr, user_data, logger):

    try:
        # Import on process
        from AzureML.score import Score
        from AzureML import Azure_Utils as AU

        # Pass in the data
        logging.setLoggerClass(logging.Logger)
        data_in = input_mgr["Data"][0]
        data = data_in.data

        # Instantiate Workspace
        ws = AU.get_workspace(
            user_data.subscription_id,
            user_data.resource_group,
            user_data.workspace_name,
        )

        # Create temp folder
        file_path = output_mgr._plugin._engine_vars.alteryx_engine.create_temp_file_name(
            extension="txt", options=0
        )
        project_path = file_path[: file_path.rfind("\\")] + "\\score"

        # Change working directory
        os.mkdir(project_path)
        os.chdir(project_path)
        logger.display_info_msg("Changed working directory: " + project_path)

        # Instantiate Score
        s = Score(ws, user_data.model_name, logger.display_info_msg)

        # Pass in data to the score
        output = s.predict(data)

        # Return dictionary with data/metadata
        data_out = output_mgr["Output"]
        data_out.data = output

    except Exception as e:
        logger.display_error_msg(str(e))


@factory.build_metadata
def define_metadata(input_mgr, output_mgr, user_data, logger):
    data_in = input_mgr["Data"][0]
    metadata = data_in.metadata

    metadata.add_column("Prediction", Sdk.FieldType.string)

    data_out = output_mgr["Output"]
    data_out.metadata = metadata


AyxPlugin = factory.generate_plugin()
